﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AntiAircraftGun
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetAntiAircraftGunsGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetAntiAircraftGunsGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="antiAircraftGun">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T antiAircraftGun)
        {
            if (_places[Count - 1] == null)
            {
                for(int i = Count - 1; i > 0; i--)
                {
                    _places[i] = _places[i - 1];
                }
                _places[0] = antiAircraftGun;
                return 0;
            }
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="antiAircraftGun">Добавляемое орудие</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T antiAircraftGun, int position)
        {
            if (position < 0 || position >= Count) return -1;
            if (_places[position] == null)
            {
                _places[position] = antiAircraftGun;
                return position;
            }
            else
            {
                if (_places[Count - 1] == null)
                {
                    for (int i = Count - 1; i > position; i--)
                    {
                        _places[i] = _places[i - 1];
                    }
                    _places[position] = antiAircraftGun;
                    return position;
                }
                return -1;
            }
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count) return null;
            if (_places[position] != null)
            {
                T removed = _places[position];
                _places[position] = null;
                if (position < Count - 1)
                {
                    for (int k = position; k < Count - 1; k++)
                    {
                        _places[k] = _places[k + 1];
                    }
                }
                return removed;
            }
            return null;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position < 0 || position >= Count) return null;
            return _places[position];
        }
    }
}
